clear all;

N = 30;
n = 0:(N-1);

if 0
   x = 2*(sin(2*pi*n/N)>0)-1;
elseif 0
   x = cos(2*pi*2*n/N);
else
   x = cos(2*pi*2.5*n/N);
end

figure(1);plot(n,x,'linewidth',3);
xlabel('n')
ylabel('x[n]');
set(gca,'fontsize',18)

X = fft(x);
k = n;

figure(2);
subplot(211);stem(k,abs(X),'linewidth',3);
xlabel('k');
ylabel('|X[k]|');
set(gca,'fontsize',18)
subplot(212);stem(k,angle(X).*(abs(X)>1e-14),'linewidth',3);
xlabel('k');
ylabel('angle(X[k])');
set(gca,'fontsize',18)

X = fftshift(fft(x));
if mod(N,2)==0 % N is even
   k = (-N/2):(N/2-1);   
else
   k = -(N-1)/2:((N-1)/2);
end
fhat = k/N;

figure(3);
subplot(211);stem(k,abs(X),'linewidth',3);
xlabel('k');
ylabel('|X[k]|');
set(gca,'fontsize',18)
subplot(212);stem(k,angle(X).*(abs(X)>1e-14),'linewidth',3);
xlabel('k');
ylabel('angle(X[k])');
set(gca,'fontsize',18)

figure(4);
subplot(211);stem(fhat,abs(X),'linewidth',3);
xlabel('$\hat f$','Interpreter','Latex');
ylabel('$|X[k]|$','Interpreter','Latex');
set(gca,'fontsize',18)
subplot(212);stem(fhat,angle(X).*(abs(X)>1e-14),'linewidth',3);
xlabel('$\hat f$','Interpreter','Latex');
ylabel('$\angle X[k]$','Interpreter','Latex');
set(gca,'fontsize',18)


%%

X = fft(x);
K = 5;
figure(5);clf
approx = zeros(size(x));

k = 0;
sinusoid(k+1,:) = cos(2*pi*k*n/N);
approx = approx + X(k+1)*exp(1i*2*pi*n*k/N);

for k = 1:K
   sinusoid(k+1,:) = cos(2*pi*k*n/N);
   approx = approx + X(k+1)*exp(1i*2*pi*n*k/N)+X(N-(k-1))*exp(-1i*2*pi*n*k/N);
      
   subplot(K,3,3*(k-1)+1);plot(n,sinusoid(k+1,:),'linewidth',3);
   hold on;
   stem(n,sinusoid(k+1,:),'linewidth',3);
   ylabel(['$\cos(2\pi n' num2str(k) '/N)$'],'Interpreter','Latex');
   set(gca,'fontsize',18)
   axis([min(n) max(n) -1.2 1.2]);
   
   subplot(K,3,3*(k-1)+2);plot(n,abs(X(k+1))*sinusoid(k+1,:),'linewidth',3);
   hold on;
   stem(n,abs(X(k+1))*sinusoid(k+1,:),'linewidth',3);
   ylabel(['$|X_' num2str(k) '|\cos(2\pi n' num2str(k) '/N)$'],'Interpreter','Latex');
   set(gca,'fontsize',18)
   axis([min(n) max(n) -1.2*max(abs(X)) 1.2*max(abs(X))]);
   
   subplot(K,3,3*(k-1)+3);plot(n,real(approx)/N,'linewidth',3);
   hold on;
   stem(n,real(approx)/N,'linewidth',3);
   hold on;plot(n,x,'k','linewidth',3);
   ylabel(['k = ' num2str(k)],'Interpreter','Latex');
   set(gca,'fontsize',18)
   %    axis([min(n) max(n) -0.2*max(abs(x)) 1.2*max(abs(x))]);
   
end
figure(5);
subplot(K,3,1);title('Harmonic Building Blocks')
subplot(K,3,2);title('Scaled Harmonic Building Blocks')
subplot(K,3,3);title('approximation using first k sinusoids');
subplot(K,3,3*(K-1)+1);xlabel('n')
subplot(K,3,3*(K-1)+2);xlabel('n')
subplot(K,3,3*(K-1)+3);xlabel('n')

return
